***************
*
*       PROM VERSION
*
*       THIS MONITOR PROGRAM IS INTENDED TO BE LOADED INTO
*       EEPROM ON THE E2 VERSION OF THE 68HC11.  THIS WILL
*       PERMIT A MINIMUM DEBUGGING SYSTEM CONSISTING ONLY OF
*       A 68HC811E2 WITH RS232 LEVEL SHIFTERS.
*
*       THIS PROGRAM WILL OPERATE CORRECTLY IF RUN FROM ANY
*       MEMORY AT FF00 - FFFF, INCLUDING INTERNAL EPROM, ROM, OR
*       ANY TYPE OF EXTERNAL MEMORY.  IT SHOULD BE NOTED THAT
*       THE USER WILL WANT THE ABILITY TO CHANGE THE VECTOR
*       TABLE, AND THEREFORE IT IS NOT RECOMMENDED THAT THIS
*       PROGRAM BE STORED IN ROM OR EEPROM.
*
*       BY:             PHILIP DEVRIES
*       VERSION:        3.0 
*       DATE:           DECEMBER 1, 1994
*       FILENAME:       6811PROM.ASM
*       COMPILED BY:    MOTOROLA FREEWARE AS11 CROSS-ASSEMBLER
*       
*       MEMORY USE     
*       BY PROGRAM:     FF00 - FFBF
*       VECTOR TABLE:   FFC0 - FFFF
*       CONTROLLER
*       REGISTER LOC:   0000 - 003F (SET BY X OFFSET)
*
*       FOR USE WITH
*       PC MONITOR 
*       VERSION:        2.X
*       FILENAME:       6811.C   
*       COMPILED BY:    QUICK-C VERSION 2 BY MICROSOFT
*
***************

***************
*
*       REGISTER USE:
*
*       REGISTER A & B  USED FOR MOVING DATA
*       REGISTER X      USED AS OFFSET TO STATUS REGISTERS
*       REGISTER Y      USED AS POINTER TO MEMORY
*                       AND AS LOCATION TO MOVE STACK POINTER
*                       TO AND FROM PC MONITOR
*       TOC5            USE AS 2 BYTE LOCATION FOR MONITOR STACK
*
***************

***************
*       Equates   
***************

TCNT    EQU     $0E             Timer counter
TOC4    EQU     $1C             OC4 Timer
TOC5    EQU     $1E             OC5 Timer
TMSK1   EQU     $22             Timer mask 1
TFLG1   EQU     $23             Timer interrupt flags 1
TMSK2   EQU     $24             Timer mask 2
TFLG2   EQU     $25             Timer interrupt flags 2
BAUD    EQU     $2B             SCI baud reg
SCCR1   EQU     $2C             SCI control1 reg
SCCR2   EQU     $2D             SCI control2 reg
SCSR    EQU     $2E             SCI status reg
SCDR    EQU     $2F             SCI data reg
BPROT   EQU     $1035           Block protect
OPTION  EQU     $1039           Configuration options
COPRST  EQU     $3A             COP reset register
PPROG   EQU     $3B             EEPROM programming register
INIT    EQU     $103D           Memory map register
CONFIG  EQU     $103F           Startup configuration

STACK   EQU     $1F             Timer OC5.  Used for monitor stack


**************
*       Config Register
*
*       If chip is 68HC811E2, map EEPROM to F800 - FFFF when reset
*
*       Enable the cop, rom, and eeprom, but not security.
*
*       Note that this is an eeprom location, that must be programmed
*       accordingly.
**************
        
        ORG     CONFIG                  
        FCB     %11111011                   

***************
*       This is the reset location for all resets, cold boot, hard reset,
*       clock monitor, and COP.  
*
*       Some of these register changes must happen in the first 64 clock
*       cycles.
*
*       Note that interrupts are disabled (including XIRQ) on reset.  
*       Internal registers are mapped to page one on reset.
***************

        ORG     $FF00
RESET  
COP
CLKMON                                  
        LDAA    #%00000000              Enable EEPROM writing
        STAA    BPROT                   
        
        LDAA    #%00010010              ADC off, IRQ level sensitive, clock
        STAA    OPTION                  monitor disabled, Cop period 262.14mS
        
        LDAA    #%00000000              Map internal Ram to 0000 and
        STAA    INIT                    registers to 0000
        
        LDAA    #'R                     Indicates reset
        BRA     INITIAL

**************
*       These vectors into the monitor program are all technically
*       interrupt service routines.  
*       
*       The initialization character stored in A is used to tell the
*       pc monitor which type of interrupt occured.
*
*       Note that the all maskable interrupts are automatically
*       disabled here.
**************
                  
TRACE   LDAA    #'S             Indicates singe step break/ timer OC5
        BRA     INITIAL  

SOFTINT LDAA    #'B             Indicates breakpoint interrupt/ SWI
        BRA     INITIAL        

ILLOP   LDAA    #'O             Indicates illegal opcode trap
        BRA     INITIAL  

TIMER   LDAA    #'T             Indicates timer interrupt
        BRA     INITIAL  

MSKINT  LDAA    #'I             Indicates maskable interrupt
        BRA     INITIAL  

NMI     LDAA    #'M             Indicates nonmaskable interrupt
        BRA     INITIAL

MONITOR LDAA    #'X             Indicates pc monitor break/ SCI    
*       BRA     INITIAL

**************
*       Initialize the offset and set up the monitor stack accordingly.
*       The user stack is temporarily stored in Y, to be recovered by
*       the monitor program with the 'S' command.
**************

INITIAL TSY                     Store the user stack in Y

        LDX     #$0000          This is the register offset

        LDAB    #STACK+1
        ABX                     Put location of OC5 into X
        TXS                     Put into stack pointer
        XGDX                    Three instructions to 
        CLRB                    restore offset
        XGDX

**************
*       Set up the serial communications port, and send the initial
*       character in from register A.
**************

SCI_INTI
        LDAB    #%00110000      Set the baud rate 9600
        STAB    BAUD,X
        
        LDAB    #%11000000      Set up number of bits serial to 8
        STAB    SCCR1,X
        
        LDAB    #%00101100      Enable SCI interupts,
        STAB    SCCR2,X         enable transmit and receive

        BSR     OUTCHAR         Send initialization character

**************
*       This is the main loop of the monitor program
**************

MAIN    BSR     INCHAR          Get instruction code from pc monitor

        CMPA    #'S             S means send X to pc monitor
        BEQ     SENDADD
        
        CMPA    #'A             A means get X from pc monitor
        BEQ     GETADDR
        
        CMPA    #'G             G means GO!
        BEQ     GO

        CLRB                    Number to increment: 0

        CMPA    #'D             D means send data to pc monitor
        BEQ     DUMPMEM         without increment
        
        CMPA    #'E             E means edit memory contents
        BEQ     EDITMEM         without increment
        
        INCB                    Number to increment: 1
        
        CMPA    #'M             M means send data to pc monitor
        BEQ     DUMPMEM         with increment

        CMPA    #'C             C means change memory contents
        BEQ     EDITMEM         with increment
        
        BRA     MAIN            Invalid instruction code is ignored

**************
*       Send data from memory location X to pc monitor.
**************

DUMPMEM LDAA    ,Y              Read data
        BSR     OUTCHAR         Send to pc monitor
        ABY                     Increment Y if B = 1
        BRA     MAIN            

**************
*       Receive data from pc monitor, save at memory location X. 
**************
                                           
EDITMEM BSR     INCHAR          Get data from pc monitor
        STAA    ,Y              Store pc monitor data in memory
        ABY                     Increment Y if B = 1
        BRA     MAIN            

**************
*       Receive address from pc monitor, save in X.
**************

GETADDR BSR     INCHAR          Get 1st character of address (LSBS)
        TAB                     Put in B, low address
        BSR     INCHAR          Get 2nd character of address (MSBS)
*                               Put in A, high address        
        XGDY                    Store address from D in Y
        BRA     MAIN  

**************
*       Send the current address in X to the pc monitor.
**************

SENDADD XGDY                    Put Y into D
        STAB    SCDR,X          Send LSB's
        BSR     OUTCHAR         Send MSB's
        XGDY                    Store address from D in Y and return
        BRA     MAIN            Store address from D in Y and return

**************                
*       This subroutine serves to start the program for both the GO and
*       the trace command.  If the command is trace, the monitor has to
*       enable the I bit of the CCR and send data $08 for the TFLG1 and
*       TMSK1 register, therebye enabling the OC5 interrupt.
*       If the command is go, the monitor must send data $00 for TFLG1
*       and TMSK1, therebye not enabling the OC5 interrupt.
**************

GO      TYS                             Load the stack pointer with the user stack
        LDD     #0034                   Use timer OC5 to generate an interrupt
        ADDD    TCNT,X                  just after the first instruction of the
        STD     TOC5,X                  user program.  
        BSET    TFLG1,X #%11111111      Clear all timer interrupt flags
        BSET    TFLG2,X #%11110000
        RTI

**************
*       This subroutine waits for an input on the SCI (RS-232)
*       and returns the data in register A.  The COP timer is
*       also reset here.
**************

INCHAR  LDAA    #%01010101              Reset COP monitor
        STAA    COPRST,X
        LDAA    #%10101010
        STAA    COPRST,X
        
        BRCLR   SCSR,X  #%00100000      INCHAR
        LDAA    SCDR,X
        RTS

**************
*       This subroutine waits for the transmitter buffer to
*       empty, then transmits the character from A.
**************

OUTCHAR BRCLR   SCSR,X  #%1100000       OUTCHAR
        STAA    SCDR,X
        RTS

**************
*       Interrupt and reset vectors.
*       This is the table location for normal modes.
**************         

         ORG    $FFC0
         FCC    " -VECTOR TABLE- "
         FCC    "      "


         ORG    $FFD6
VSCI     FDB    MONITOR
VSPI     FDB    TIMER
VPAIE    FDB    TIMER
VPAO     FDB    TIMER
VTOF     FDB    TIMER
VTOC5    FDB    TRACE
VTOC4    FDB    TIMER
VTOC3    FDB    TIMER
VTOC2    FDB    TIMER
VTOC1    FDB    TIMER
VTIC3    FDB    TIMER
VTIC2    FDB    TIMER
VTIC1    FDB    TIMER
VRTI     FDB    TIMER
VIRQ     FDB    MSKINT 
VXIRQ    FDB    NMI    
VSWI     FDB    SOFTINT
VILLOP   FDB    ILLOP  
VCOP     FDB    COP
VCLM     FDB    CLKMON
VRST     FDB    RESET 
